#!/usr/bin/perl

##########################################################
#
# This software is distributed AS IS, as open source
# freeware PERL code. There is no implicit
# guarantee that this software will serve your needs,
# work on your software/hardware nor that I will
# provide users any sort of assistance.
#
# Free usage is granted for all those purposes that
# do not require any sort of fee to gain access to.
#
# Version 0.1, 2001-10-10
$version = "0.1";
$version_date = "2001-10-10";
# Idea (c) of Paolo Cravero, ik1zyw at qsl dot net
#
##########################################################
#
# REQUIREMENTS
# - perl 5.x or higher
#   + perl IO::File module
# - lynx 2.8.4 or higher
# - internet connection :-)
#
##########################################################

use IO::File;

# This is the base URL, the starting point for the news list
$baseURL = "http://www.televideo.rai.it/nazionale/homenaz.asp";
#$baseURL = "homenaz.asp.html"; #for offline testing

# When to invalidate a page if shorter than... bytes
$limitlength = 1000; #bytes

# How many levels should be retrieved (news are split in
# several pages with few news each, newest on top)
# The program stops when a news is already on disk
$level = 12; #must be higher or equal to 1

# Set the connection timeout to lynx.
# Lynx stops trying the page after $timeout seconds.
$timeout = 5; #seconds

# Set the break between page retrieval in seconds
$break = 0; #seconds

# Get the page with news list
sub get_home {

    # run lynx and get a dump of the page
    # lynx's dump actually returns at the bottom the list of
    # URLs references in the page, so we grep only those that
    # have a NewsID
    print "Retrieving news list... ";
    $_ = `lynx -connect_timeout=$timeout -width=99999 -dump -force_html "$homeURL"   | grep NewsID`;

    # if the page is too short, there might have been a problem
    if (length($_)<$limitlength)
    {
	print "Error reading Televideo homepage ($homeURL)\nAbort...\n";
	exit 1;
    }
    print "done\n";
}


# Genereates a unique list of links to news pages
sub split_news() {

    my $junk;
    my $url;
    my $pair;

    # make an array out of the list of URLs provided by lynx
    @allclean = split(/\n/,$_);
    foreach $pair (@allclean) {
	($junk, $url) = split(/\. /,$pair);
	$URL{$url}=1;
    }

}


# Connect to the news server and retieve a news page
sub get_news() {

    # we use lynx to get the page, the HTML source of it
    $_ = `lynx -connect_timeout=$timeout -source "$key"`;

    # if the page is too short, there might have been a problem
    # we just skip this news entry
    if (length($_)<$limitlength)
    {
	print "Error reading Televideo newspage ($key)\nSkipping...\n";
	return 0;
    }
    return 1;
}


# Save the news in the ./news/ directory
# If the news entry already exists, don't save
# and warn the application
sub save_news() {

    my $junk;
    my $news_number;
    my $file;

    # get the news entry ID
    ($junk, $news_number) = split(/NewsID=/,$key);
    # check if we have it already
    if (open(FH," news/$news_number.html")) {
	return 1;
    }

    # if we don't, then save it
    print "I am saving news $news_number... ";
    $file = new IO::File "> news/$news_number.html";
    print $file $_;
    print "done!\n";
    $file->close();
    
    return 0;
}


########
# MAIN #
########

print "\n      Televideo RAI Ultim'ora Retriever\n";
print "          Version $version ($version_date)\n";
print "            by Paolo Cravero (c)\n\n\n";

RETR: for (my $i=1;$i<=$level;$i++) {

    @keys = ();
    @allclean = ();
    %URL = ();

    # compose the URL for retrieving multiple pages
    $homeURL = $baseURL.+ "?currentStep=$i&Sezione=Ultimora\n";
    # get the page
    &get_home();

    # split the news into an array
    &split_news();

    # sort the news with newest first
    @keys = sort {$b cmp $a} keys %URL;

    # retrieve all and only new news
    foreach $key (@keys) {
	if (&get_news() ne 0) {
	    if (&save_news()) {
		last RETR;
	    }
	sleep ($break); # wait before retrieving next page
	}
    }

}

print "\nAll new news downloaded.\nExiting...\n\n";
exit 0;

